/*
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SINGLETON_H
#define SINGLETON_H

#include <QObject>
#include <QCoreApplication>

template<typename T>
class Singleton
{
public:
    static T *instance() {
        static T instance;
        static bool moveToMainThread = true;

        if (QtPrivate::AreArgumentsCompatible<T, QObject>::value && moveToMainThread) {
            moveToMainThread = false;

            QObject *object = dynamic_cast<QObject*>(&instance);

            object->moveToThread(qApp->thread());
        }

        return &instance;
    }

private:
    Singleton();
    ~Singleton();
    Singleton(const Singleton &);
    Singleton & operator = (const Singleton &);
};

#endif // SINGLETON_H
